SSIB = SSIB or {
	_default_localization_path = ModPath .. "localization/english.json",
	_menu_path = ModPath .. "menu/options.json",
	_save_path = SavePath .. "separate_shout_interaction_binds.json",
	settings = nil,
	default_settings = {
		interact_enabled = true,
		interact_use_modifier = false,
		intimidate_enabled = true,
		intimidate_use_modifier = false,
		call_enabled = true,
		call_use_modifier = false,
		mark_enabled = true,
		mark_use_modifier = false
	}
}
SSIB.settings = table.deep_map_copy(SSIB.default_settings)

function SSIB:c_log(str)
	if OffyLib then
		OffyLib:c_log(str)
	end
end


function SSIB:CheckInteractWanted()
	return HoldTheKey:Keybind_Held("ssib_interact")
end

function SSIB:CheckIntimidateWanted()
	return HoldTheKey:Keybind_Held("ssib_intimidate")
end

function SSIB:CheckCallWanted()
	return HoldTheKey:Keybind_Held("ssib_call")
end

function SSIB:CheckMarkWanted()
	return HoldTheKey:Keybind_Held("ssib_mark")
end


function SSIB:DoActionInteract()
	local t = TimerManager:game():time()
	local player = managers.player:local_player()
	if alive(player) then
		local mov_ext = player:movement()
		local state = mov_ext and mov_ext:current_state()
		if state then
			local input = state._input
			self:_DoActionInteract(state,t,input)
		end
	end
end

function SSIB:_DoActionInteract(state,t,input)
	if not state:_action_interact_forbidden() then
		local new_action, timer, interact_object = state._interaction:interact(state._unit, input.data, PlayerHand.RIGHT)

		if new_action then
			state:_play_interact_redirect(t, input)
		end

		if timer then
			new_action = true

			self:c_log("Interact " .. tostring(new_action))
			state._ext_camera:camera_unit():base():set_limits(80, 50)
			state:_start_action_interact(t, input, timer, interact_object)
		end
		
		return new_action
	end
end


function SSIB:DoActionIntimidate()
	local t = TimerManager:game():time()
	local player = managers.player:local_player()
	if alive(player) then
		local mov_ext = player:movement()
		local state = mov_ext and mov_ext:current_state()
		if state then
			local input = state._input
			self:_DoActionIntimidate(state,t,input)
		end
	end
end

function SSIB:_DoActionIntimidate(state,t,input)
	local is_whisper_mode = managers.groupai:state():whisper_mode()
	local intimidate_enemies = true
	local intimidate_civilians = true
	local intimidate_teammates = false
	local only_special_enemies = false
	local intimidate_escorts = true
	local intimidation_amount = nil
	local primary_only = nil --only allow enemy intimidation, not ai shouting
	local detect_only = false
	local secondary = false --ai calling
	
	local secondary_delay = tweak_data.team_ai.stop_action.delay
	local voice_type, plural, prime_target
	
	if not state._intimidate_t or tweak_data.player.movement_state.interaction_delay < t - state._intimidate_t then
		if not state:_action_interact_forbidden() then
			self:c_log("Intimidate " .. tostring(voice_type))
			voice_type, plural, prime_target = self:GetUnitIntimidationAction(state,
				intimidate_enemies,
				intimidate_civilians,
				intimidate_teammates,
				only_special_enemies,
				intimidate_escorts,
				intimidation_amount,
				primary_only,
				detect_only,
				secondary
			)
		--[[
			voice_type, plural, prime_target = state:_get_unit_intimidation_action(
				intimidate_enemies,
				intimidate_civilians,
				intimidate_teammates,
				only_special_enemies,
				intimidate_escorts,
				intimidation_amount,
				primary_only,
				detect_only,
				secondary
			)
			--]]
			state._start_intimidate = false
		else
			state:_interupt_action_interact()
		end
	end
	
	if not voice_type then
		--no valid action
		return
	end
	
	local interact_type,sound_name = self:GetRedirect(state,t,voice_type,plural,prime_target)
	state:_do_action_intimidate(t,interact_type,sound_name,is_whisper_mode)
	
	return true
end


function SSIB:DoActionCall()
	local t = TimerManager:game():time()
	local player = managers.player:local_player()
	if alive(player) then
		local mov_ext = player:movement()
		local state = mov_ext and mov_ext:current_state()
		if state then
			local input = state._input
			self:_DoActionCall(state,t,input)
		end
	end
end

function SSIB:_DoActionCall(state,t,input)
	local is_whisper_mode = managers.groupai:state():whisper_mode()
	
	local intimidate_enemies = false
	local intimidate_civilians = false
	local intimidate_teammates = true
	local only_special_enemies = false
	local intimidate_escorts = true
	local intimidation_amount = nil
	local primary_only = nil
	local detect_only = false
	local secondary = false
	
	local secondary_delay = tweak_data.team_ai.stop_action.delay
	
	local voice_type, plural, prime_target 
	if not state._intimidate_t or tweak_data.player.movement_state.interaction_delay < t - state._intimidate_t then
		
		if not state:_action_interact_forbidden() then
			voice_type, plural, prime_target = state:_get_unit_intimidation_action(
				intimidate_enemies,
				intimidate_civilians,
				intimidate_teammates,
				only_special_enemies,
				intimidate_escorts,
				intimidation_amount,
				primary_only,
				detect_only,
				secondary
			)
			self:c_log("Call " .. tostring(voice_type))
			state._start_intimidate = false
		else
			state:_interupt_action_interact()
			return
		end
	end
	
	if not voice_type then
		return
	end
	
	local interact_type,sound_name = self:GetRedirect(state,t,voice_type,plural,prime_target)
	state:_do_action_intimidate(t,interact_type,sound_name,is_whisper_mode)
	
	return true
end


function SSIB:DoActionMark()
	local t = TimerManager:game():time()
	local player = managers.player:local_player()
	if alive(player) then
		local mov_ext = player:movement()
		local state = mov_ext and mov_ext:current_state()
		if state then
			local input = state._input
			self:_DoActionMark(state,t,input)
		end
	end
end

function SSIB:_DoActionMark(state,t,input)
	local is_whisper_mode = managers.groupai:state():whisper_mode()

	local intimidate_enemies = true
	local intimidate_civilians = true
	local intimidate_teammates = false
	local only_special_enemies = not is_whisper_mode
	local intimidate_escorts = false
	local intimidation_amount = nil
	local primary_only = true --only allow enemy intimidation, not ai shouting
	local detect_only = false
	local secondary = false --ai calling
	
	local secondary_delay = tweak_data.team_ai.stop_action.delay
	
	local voice_type, plural, prime_target
	if not state:_action_interact_forbidden() then
		voice_type, plural, prime_target = state:_get_unit_intimidation_action(
			intimidate_enemies,
			intimidate_civilians,
			intimidate_teammates,
			only_special_enemies,
			intimidate_escorts,
			intimidation_amount,
			primary_only,
			detect_only,
			secondary
		)
		self:c_log("Mark " .. tostring(voice_type))

		state._start_intimidate = false
	else
		state:_interupt_action_interact()
	end
	
	if not voice_type then
		return
	end
	
	local interact_type,sound_name = self:GetRedirect(state,t,voice_type,plural,prime_target)
	
	state:_do_action_intimidate(t,interact_type,sound_name,is_whisper_mode)
	return true
end


--very similar to PlayerStandard:_get_intimidation_action() 
--except that it allows filtering for certain actions
--currently only used for intimidation keybind
function SSIB:GetIntimidationAction(state, prime_target, char_table, amount, primary_only, detect_only, secondary, action_type)
	local voice_type, new_action, plural = nil
	local unit_type_enemy = 0
	local unit_type_civilian = 1
	local unit_type_teammate = 2
	local unit_type_camera = 3
	local unit_type_turret = 4
	local is_whisper_mode = managers.groupai:state():whisper_mode()
	
	if prime_target then
		if action_type == "call" then
			if prime_target.unit_type == unit_type_teammate then
				local is_human_player, record = nil

				if not detect_only then
					record = managers.groupai:state():all_criminals()[prime_target.unit:key()]

					if record.ai then
						if not prime_target.unit:brain():player_ignore() then
							prime_target.unit:movement():set_cool(false)
							prime_target.unit:brain():on_long_dis_interacted(0, state._unit, secondary)
						end
					else
						is_human_player = true
					end
				end

				local amount = 0
				local current_state_name = state._unit:movement():current_state_name()

				if current_state_name ~= "arrested" and current_state_name ~= "bleed_out" and current_state_name ~= "fatal" and current_state_name ~= "incapacitated" then
					local rally_skill_data = state._ext_movement:rally_skill_data()

					if rally_skill_data and mvector3.distance_sq(state._pos, record.m_pos) < rally_skill_data.range_sq then
						local needs_revive, is_arrested, action_stop = nil

						if not secondary then
							if prime_target.unit:base().is_husk_player then
								is_arrested = prime_target.unit:movement():current_state_name() == "arrested"
								needs_revive = prime_target.unit:interaction():active() and prime_target.unit:movement():need_revive() and not is_arrested
							else
								is_arrested = prime_target.unit:character_damage():arrested()
								needs_revive = prime_target.unit:character_damage():need_revive()
							end

							if needs_revive and managers.player:has_enabled_cooldown_upgrade("cooldown", "long_dis_revive") then
								voice_type = "revive"

								managers.player:disable_cooldown_upgrade("cooldown", "long_dis_revive")
							elseif not is_arrested and not needs_revive and rally_skill_data.morale_boost_delay_t and rally_skill_data.morale_boost_delay_t < managers.player:player_timer():time() then
								voice_type = "boost"
								amount = 1
							end
						end
					end
				end

				if is_human_player then
					prime_target.unit:network():send_to_unit({
						"long_dis_interaction",
						prime_target.unit,
						amount,
						state._unit,
						secondary or false
					})
				end

				voice_type = voice_type or secondary and "ai_stay" or "come"
				plural = false
			end
		else
			local prime_target_key = prime_target.unit:key()

			if prime_target.unit_type == unit_type_enemy then
				plural = false
				
				if action_type == "intimidate" then
					if prime_target.unit:anim_data().hands_back then
						voice_type = "cuff_cop"
					elseif prime_target.unit:anim_data().surrender then
						voice_type = "down_cop"
					elseif not (prime_target.unit:movement():cool() or prime_target.unit:base():char_tweak().silent_priority_shout) then
						voice_type = "stop_cop"
					end
				elseif action_type == "mark" then
					if is_whisper_mode and prime_target.unit:movement():cool() and prime_target.unit:base():char_tweak().silent_priority_shout then
						voice_type = "mark_cop_quiet"
					elseif prime_target.unit:base():char_tweak().priority_shout then
						voice_type = "mark_cop"
					end
				end
			elseif prime_target.unit_type == unit_type_camera then
				if action_type == "mark" then
					if not prime_target.unit or not prime_target.unit:base() or not prime_target.unit:base().is_friendly then
						plural = false
						voice_type = "mark_camera"
					end
				end
			elseif prime_target.unit_type == unit_type_turret then
				if action_type == "mark" then
					plural = false
					voice_type = "mark_turret"
				end
			elseif prime_target.unit:base():char_tweak().is_escort then
				if action_type == "intimidate" then
					plural = false
					local e_guy = prime_target.unit

					if e_guy:anim_data().move then
						voice_type = "escort_keep"
					elseif e_guy:anim_data().panic then
						voice_type = "escort_go"
					else
						voice_type = prime_target.unit:base():char_tweak().speech_escort or "escort"
					end
				end
			else
				if action_type == "call" then
					if prime_target.unit:movement():stance_name() == "cbt" and prime_target.unit:anim_data().stand then
						voice_type = "come"
					elseif prime_target.unit:anim_data().move then
						voice_type = "stop"
					elseif prime_target.unit:anim_data().drop then
						voice_type = "down_stay"
					else
						voice_type = "down"
					end

					local num_affected = 0

					for _, char in pairs(char_table) do
						if char.unit_type == unit_type_civilian then
							if voice_type == "stop" and char.unit:anim_data().move then
								num_affected = num_affected + 1
							elseif voice_type == "down_stay" and char.unit:anim_data().drop then
								num_affected = num_affected + 1
							elseif voice_type == "down" and not char.unit:anim_data().move and not char.unit:anim_data().drop then
								num_affected = num_affected + 1
							end
						end
					end

					if num_affected > 1 then
						plural = true
					else
						plural = false
					end
				end
			end

			local max_inv_wgt = 0

			for _, char in pairs(char_table) do
				if max_inv_wgt < char.inv_wgt then
					max_inv_wgt = char.inv_wgt
				end
			end

			if max_inv_wgt < 1 then
				max_inv_wgt = 1
			end

			if detect_only or action_type == "call" then
				voice_type = "come"
			elseif action_type == "intimidate" then
				for _, char in pairs(char_table) do
					if char.unit_type ~= unit_type_camera and char.unit_type ~= unit_type_teammate and (not is_whisper_mode or not char.unit:movement():cool()) then
						if char.unit_type == unit_type_civilian then
							amount = (amount or tweak_data.player.long_dis_interaction.intimidate_strength) * managers.player:upgrade_value("player", "civ_intimidation_mul", 1) * managers.player:team_upgrade_value("player", "civ_intimidation_mul", 1)
						end

						if prime_target_key == char.unit:key() then
							voice_type = char.unit:brain():on_intimidated(amount or tweak_data.player.long_dis_interaction.intimidate_strength, state._unit) or voice_type
						elseif not primary_only and char.unit_type ~= unit_type_enemy then
							char.unit:brain():on_intimidated((amount or tweak_data.player.long_dis_interaction.intimidate_strength) * char.inv_wgt / max_inv_wgt, state._unit)
						end
					end
				end
			end
		end
	end

	return voice_type, plural, prime_target
end

--substitute for _get_unit_intimidation_action() that is identical
--except that it returns SSIB:GetIntimidationAction() instead of PlayerStandard:_get_intimidation_action()
function SSIB:GetUnitIntimidationAction(state,intimidate_enemies,intimidate_civilians,intimidate_teammates,only_special_enemies,intimidate_escorts,intimidation_amount,primary_only,detect_only,secondary)
	local char_table = {}
	local unit_type_enemy = 0
	local unit_type_civilian = 1
	local unit_type_teammate = 2
	local unit_type_camera = 3
	local unit_type_turret = 4
	local cam_fwd = state._ext_camera:forward()
	local my_head_pos = state._ext_movement:m_head_pos()

	if _G.IS_VR then
		local hand_unit = state._unit:hand():hand_unit(state._interact_hand)

		if hand_unit:raycast("ray", hand_unit:position(), my_head_pos, "slot_mask", 1) then
			return
		end

		cam_fwd = hand_unit:rotation():y()
		my_head_pos = hand_unit:position()
	end

	local spotting_mul = managers.player:upgrade_value("player", "marked_distance_mul", 1)
	local range_mul = managers.player:upgrade_value("player", "intimidate_range_mul", 1) * managers.player:upgrade_value("player", "passive_intimidate_range_mul", 1)
	local intimidate_range_civ = tweak_data.player.long_dis_interaction.intimidate_range_civilians * range_mul
	local intimidate_range_ene = tweak_data.player.long_dis_interaction.intimidate_range_enemies * range_mul
	local highlight_range = tweak_data.player.long_dis_interaction.highlight_range * range_mul * spotting_mul
	local intimidate_range_teammates = tweak_data.player.long_dis_interaction.intimidate_range_teammates

	if intimidate_enemies then
		local enemies = managers.enemy:all_enemies()

		for u_key, u_data in pairs(enemies) do
			if state._unit:movement():team().foes[u_data.unit:movement():team().id] and not u_data.unit:anim_data().hands_tied and not u_data.unit:anim_data().long_dis_interact_disabled and (not u_data.unit:character_damage() or not u_data.unit:character_damage():dead()) and (u_data.char_tweak.priority_shout or not only_special_enemies) then
				if managers.groupai:state():whisper_mode() then
					if u_data.char_tweak.silent_priority_shout and u_data.unit:movement():cool() then
						state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_enemy, highlight_range, false, false, 0.01, my_head_pos, cam_fwd)
					elseif not u_data.unit:movement():cool() then
						state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_enemy, intimidate_range_ene, false, false, 100, my_head_pos, cam_fwd, nil, "ai_vision mover")
					end
				elseif u_data.char_tweak.priority_shout then
					state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_enemy, highlight_range, false, false, 0.01, my_head_pos, cam_fwd)
				else
					state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_enemy, intimidate_range_ene, false, false, 100, my_head_pos, cam_fwd, nil, "ai_vision mover")
				end
			end
		end
	end

	if intimidate_civilians then
		local civilians = managers.enemy:all_civilians()

		for u_key, u_data in pairs(civilians) do
			if alive(u_data.unit) and u_data.unit:in_slot(21) and not u_data.unit:movement():cool() and not u_data.unit:anim_data().long_dis_interact_disabled then
				local is_escort = u_data.char_tweak.is_escort

				if not is_escort or intimidate_escorts then
					local dist = is_escort and 300 or intimidate_range_civ
					local prio = is_escort and 100000 or 0.001

					state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_civilian, dist, false, false, prio, my_head_pos, cam_fwd)
				end
			end
		end
	end

	if intimidate_teammates and not managers.groupai:state():whisper_mode() then
		local criminals = managers.groupai:state():all_char_criminals()

		for u_key, u_data in pairs(criminals) do
			local added = nil

			if u_key ~= state._unit:key() then
				local rally_skill_data = state._ext_movement:rally_skill_data()

				if rally_skill_data and rally_skill_data.long_dis_revive and mvector3.distance_sq(state._pos, u_data.m_pos) < rally_skill_data.range_sq then
					local needs_revive = nil

					if u_data.unit:base().is_husk_player then
						needs_revive = u_data.unit:interaction():active() and u_data.unit:movement():need_revive() and u_data.unit:movement():current_state_name() ~= "arrested"
					else
						needs_revive = u_data.unit:character_damage():need_revive()
					end

					if needs_revive then
						added = true

						state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_teammate, intimidate_range_teammates, true, true, 5000, my_head_pos, cam_fwd)
					end
				end
			end

			if not added and not u_data.is_deployable and not u_data.unit:movement():downed() and not u_data.unit:base().is_local_player and not u_data.unit:anim_data().long_dis_interact_disabled then
				state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_teammate, intimidate_range_teammates, true, not secondary, 0.01, my_head_pos, cam_fwd)
			end
		end
	end

	if intimidate_enemies and intimidate_teammates then
		local enemies = managers.enemy:all_enemies()

		for u_key, u_data in pairs(enemies) do
			if u_data.unit:movement():team() and u_data.unit:movement():team().id == "criminal1" and not u_data.unit:movement():cool() and not u_data.unit:anim_data().long_dis_interact_disabled then
				local is_escort = u_data.char_tweak.is_escort

				if not is_escort or intimidate_escorts then
					local dist = is_escort and 300 or intimidate_range_civ
					local prio = is_escort and 100000 or 0.001

					state:_add_unit_to_char_table(char_table, u_data.unit, unit_type_civilian, dist, false, false, prio, my_head_pos, cam_fwd)
				end
			end
		end
	end

	if intimidate_enemies then
		if managers.groupai:state():whisper_mode() then
			for _, unit in ipairs(SecurityCamera.cameras) do
				if alive(unit) and unit:enabled() and not unit:base():destroyed() then
					local dist = 2000
					local prio = 0.001

					state:_add_unit_to_char_table(char_table, unit, unit_type_camera, dist, false, false, prio, my_head_pos, cam_fwd, {
						unit
					})
				end
			end
		end

		local turret_units = managers.groupai:state():turrets()

		if turret_units then
			for _, unit in pairs(turret_units) do
				if alive(unit) and unit:movement():team().foes[state._ext_movement:team().id] then
					state:_add_unit_to_char_table(char_table, unit, unit_type_turret, 2000, false, false, 0.01, my_head_pos, cam_fwd, {
						unit
					})
				end
			end
		end
	end

	local prime_target = state:_get_interaction_target(char_table, my_head_pos, cam_fwd)

	return self:GetIntimidationAction(state, prime_target, char_table, intimidation_amount, primary_only, detect_only, secondary, "intimidate")
end

--given shout result, play voiceline and fp anim
--(_start_action_intimidate substitute)
function SSIB:GetRedirect(state,t,voice_type,plural,prime_target)
	if not state._intimidate_t or tweak_data.player.movement_state.interaction_delay < t - state._intimidate_t then
		if prime_target and prime_target.unit and prime_target.unit.base and (prime_target.unit:base().unintimidateable or prime_target.unit:anim_data() and prime_target.unit:anim_data().unintimidateable) then
			return
		end

		local interact_type, sound_name = nil
		local sound_suffix = plural and "plu" or "sin"

		if voice_type == "stop" then
			interact_type = "cmd_stop"
			sound_name = "f02x_" .. sound_suffix
		elseif voice_type == "stop_cop" then
			interact_type = "cmd_stop"
			sound_name = "l01x_" .. sound_suffix
		elseif voice_type == "mark_cop" or voice_type == "mark_cop_quiet" then
			interact_type = "cmd_point"

			if voice_type == "mark_cop_quiet" then
				sound_name = tweak_data.character[prime_target.unit:base()._tweak_table].silent_priority_shout .. "_any"
			else
				sound_name = tweak_data.character[prime_target.unit:base()._tweak_table].priority_shout .. "x_any"
				sound_name = managers.modifiers:modify_value("PlayerStandart:_start_action_intimidate", sound_name, prime_target.unit)
			end

			if managers.player:has_category_upgrade("player", "special_enemy_highlight") then
				prime_target.unit:contour():add(managers.player:get_contour_for_marked_enemy(), true, managers.player:upgrade_value("player", "mark_enemy_time_multiplier", 1))
				managers.network:session():send_to_peers_synched("spot_enemy", prime_target.unit)
			end
		elseif voice_type == "down" then
			interact_type = "cmd_down"
			sound_name = "f02x_" .. sound_suffix
			state._shout_down_t = t
		elseif voice_type == "down_cop" then
			interact_type = "cmd_down"
			sound_name = "l02x_" .. sound_suffix
		elseif voice_type == "cuff_cop" then
			interact_type = "cmd_down"
			sound_name = "l03x_" .. sound_suffix
		elseif voice_type == "down_stay" then
			interact_type = "cmd_down"

			if state._shout_down_t and t < state._shout_down_t + 2 then
				sound_name = "f03b_any"
			else
				sound_name = "f03a_" .. sound_suffix
			end
		elseif voice_type == "come" then
			interact_type = "cmd_come"
			local static_data = managers.criminals:character_static_data_by_unit(prime_target.unit)

			if static_data then
				local character_code = static_data.ssuffix
				sound_name = "f21" .. character_code .. "_sin"
			else
				sound_name = "f38_any"
			end
		elseif voice_type == "revive" then
			interact_type = "cmd_get_up"
			local static_data = managers.criminals:character_static_data_by_unit(prime_target.unit)

			if not static_data then
				return
			end

			local character_code = static_data.ssuffix
			sound_name = "f36x_any"

			if math.random() < state._ext_movement:rally_skill_data().revive_chance then
				prime_target.unit:interaction():interact(state._unit)
			end

			state._ext_movement:rally_skill_data().morale_boost_delay_t = managers.player:player_timer():time() + (state._ext_movement:rally_skill_data().morale_boost_cooldown_t or 3.5)
		elseif voice_type == "boost" then
			interact_type = "cmd_gogo"
			local static_data = managers.criminals:character_static_data_by_unit(prime_target.unit)

			if not static_data then
				return
			end

			local character_code = static_data.ssuffix
			sound_name = "g18"
			state._ext_movement:rally_skill_data().morale_boost_delay_t = managers.player:player_timer():time() + (state._ext_movement:rally_skill_data().morale_boost_cooldown_t or 3.5)
		elseif voice_type == "escort" then
			interact_type = "cmd_point"
			sound_name = "f41_" .. sound_suffix
		elseif voice_type == "escort_keep" or voice_type == "escort_go" then
			interact_type = "cmd_point"
			sound_name = "f40_any"
		elseif voice_type == "bridge_codeword" then
			sound_name = "bri_14"
			interact_type = "cmd_point"
		elseif voice_type == "bridge_chair" then
			sound_name = "bri_29"
			interact_type = "cmd_point"
		elseif voice_type == "undercover_interrogate" then
			sound_name = "f46x_any"
			interact_type = "cmd_point"
		elseif voice_type == "undercover_escort" then
			sound_name = "f41_any"
			interact_type = "cmd_point"
		elseif voice_type == "mark_camera" then
			sound_name = "f39_any"
			interact_type = "cmd_point"

			prime_target.unit:contour():add("mark_unit", true, managers.player:upgrade_value("player", "mark_enemy_time_multiplier", 1))
		elseif voice_type == "mark_turret" then
			sound_name = "f44x_any"
			interact_type = "cmd_point"
			local type = prime_target.unit:base().get_type and prime_target.unit:base():get_type()

			prime_target.unit:contour():add(managers.player:get_contour_for_marked_enemy(type), true, managers.player:upgrade_value("player", "mark_enemy_time_multiplier", 1))
		elseif voice_type == "ai_stay" then
			sound_name = "f48x_any"
			interact_type = "cmd_stop"
		end
		
		if state._unit:movement():current_state_name() == "mask_off" then
			--in casing mode, do not play a fp redirect
			interact_type = nil
		end
		
		return interact_type,sound_name
	end
end

-- I/O

function SSIB:LoadSettings()
	local file = io.open(self._save_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
			self.settings[k] = v
		end
	else
		self:SaveSettings()
	end
end

function SSIB:SaveSettings()
	local file = io.open(self._save_path,"w+")
	if file then
		file:write(json.encode(self.settings))
		file:close()
	end
end

Hooks:Add("LocalizationManagerPostInit", "ssib_LocalizationManagerPostInit", function(loc)
	if not BeardLib then
		loc:load_localization_file(SSIB._default_localization_path)
	end
end)

Hooks:Add("MenuManagerInitialize", "ssib_MenuManagerInitialize", function(menu_manager)
	
	SSIB:LoadSettings()
	
	MenuCallbackHandler.callback_ssib_interact_enabled = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.interact_enabled = value
		SSIB:SaveSettings()
	end
	
	MenuCallbackHandler.callback_ssib_interact_use_modifier = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.interact_use_modifier = value
		SSIB:SaveSettings()
	end
	
	MenuCallbackHandler.callback_ssib_interact_keybind = function(self,item)
		if not SSIB.settings.interact_use_modifier then
			SSIB:DoActionInteract()
		end
	end
	
	MenuCallbackHandler.callback_ssib_intimidate_enabled = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.intimidate_enabled = value
		SSIB:SaveSettings()
	end
	
	MenuCallbackHandler.callback_ssib_intimidate_use_modifier = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.intimidate_use_modifier = value
		SSIB:SaveSettings()
	end
	
	MenuCallbackHandler.callback_ssib_intimidate_keybind = function(self,item)
		--intimidate
		if not SSIB.settings.intimidate_use_modifier then
			SSIB:DoActionIntimidate()
		end
	end
	
	MenuCallbackHandler.callback_ssib_call_enabled = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.call_enabled = value
		SSIB:SaveSettings()
	end
	
	MenuCallbackHandler.callback_ssib_call_use_modifier = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.call_use_modifier = value
		SSIB:SaveSettings()
	end
	
	MenuCallbackHandler.callback_ssib_call_keybind = function(self,item)
		--call teammates
		if not SSIB.settings.call_use_modifier then
			SSIB:DoActionCall()
		end
	end
	
	MenuCallbackHandler.callback_ssib_mark_enabled = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.mark_enabled = value
		SSIB:SaveSettings()
	end
	MenuCallbackHandler.callback_ssib_mark_use_modifier = function(self,item)
		local value = item:value() == "on"
		SSIB.settings.mark_use_modifier = value
		SSIB:SaveSettings()
	end
	MenuCallbackHandler.callback_ssib_mark_keybind = function(self,item)
		--spot special enemies
		if not SSIB.settings.mark_use_modifier then
			SSIB:DoActionMark()
		end
	end
	
	if HoldTheKey then
		HoldTheKey:Add_Keybind("ssib_interact")
		HoldTheKey:Add_Keybind("ssib_intimidate")
		HoldTheKey:Add_Keybind("ssib_call")
		HoldTheKey:Add_Keybind("ssib_mark")
	end
	
	MenuHelper:LoadFromJsonFile(SSIB._menu_path, SSIB, SSIB.settings)
end)